﻿//-----------------------------------------------------------------------
// <copyright file="SpatialProjectionSample.xaml.cs" company="U2U Consult">
//     http://www.U2UConsult.com
// </copyright>
// <summary>Defines the SpatialProjectionSample class.</summary>
//-----------------------------------------------------------------------

namespace U2UConsult.DockOfTheBay
{
    using System;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows.Shapes;
    using Microsoft.SqlServer.Types; // Add Reference !!!
    using SQLSpatialTools; // Add Reference !!!

    /// <summary>
    /// Demonstrates projecting Spatial Data via 
    /// SqlSpatialTools (http://sqlspatialtools.codeplex.com/). 
    /// </summary>
    public partial class SpatialProjectionSample : Window
    {
        /// <summary>
        /// Initializes a new instance of the SpatialProjectionSample class.
        /// </summary>
        public SpatialProjectionSample()
        {
            InitializeComponent();

            this.ProjectionTypesCombo.SelectedIndex = 0;
        }

        /// <summary>
        /// Evaluates and renders the WKT.
        /// </summary>
        /// <param name="sender">Sender of the Event (DrawButton).</param>
        /// <param name="e">Event Arguments.</param>
        private void DrawButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                // Read Well-Known Text
                SqlGeometry sqlGeometry = SqlGeometry.Parse(this.GeometryTextBox.Text);

                // Transform to Path
                Path path = sqlGeometry.AsPath();

                // Basic Properties
                path.Stroke = Brushes.LightGray;
                path.StrokeThickness = 1;

                // path.Effect = new DropShadowEffect() { Direction = 225, ShadowDepth = 2 };
                // Polygons only ...
                path.Fill = Brushes.DarkGreen;

                // Automate Translation & Inflation
                path.Stretch = Stretch.Uniform;

                // Display
                this.DrawingCanvas.Child = path;
            }
            catch (Exception ex)
            {
                // Input for Drawing not valid
                this.DrawingCanvas.Child = new TextBlock()
                {
                    TextWrapping = TextWrapping.Wrap,
                    MaxHeight = 128,
                    VerticalAlignment = VerticalAlignment.Top,
                    Foreground = Brushes.Red,
                    Text = ex.Message
                };
            }
        }

        /// <summary>
        /// Applies the requested Projection, and updates the GUI.
        /// </summary>
        /// <param name="sender">Sender of the Event (ProjectButton).</param>
        /// <param name="e">Event Arguments.</param>
        private void ProjectButton_Click(object sender, RoutedEventArgs e)
        {
            SqlProjection p;

            try
            {
                // Read as SqlGeography
                SqlGeography sqlGeography = SqlGeography.Parse(this.GeometryTextBox.Text);

                // Initialize Projection from GUI Parameters
                p = this.GetProjection();

                SqlGeometry sqlGeometry = p.Project(sqlGeography);

                // UTM-ish
                // sqlGeometry = AffineTransform.Scale(6378137.0, 6378137.0).Apply(sqlGeometry);
                // sqlGeometry = AffineTransform.Translate(500000, 0).Apply(sqlGeometry);

                // Transform to Path
                Path path = sqlGeometry.AsPath();

                // Basic Properties
                path.Stroke = Brushes.LightGray;
                path.StrokeThickness = 1;

                // path.Effect = new DropShadowEffect() { Direction = 225, ShadowDepth = 1 };
                // Polygons only ...
                path.Fill = Brushes.DarkRed;

                // Automate Translation & Inflation
                path.Stretch = Stretch.Uniform;

                this.ProjectedDrawing.Child = path;

                this.ProjectedTextBox.Text =
                    sqlGeometry.STAsText().ToSqlString().Value;
            }
            catch (Exception ex)
            {
                // Input for Projection not valid
                this.ProjectedDrawing.Child = new TextBlock()
                {
                    TextWrapping = TextWrapping.Wrap,
                    MaxHeight = 128,
                    VerticalAlignment = VerticalAlignment.Top,
                    Foreground = Brushes.Red,
                    Text = ex.Message
                };
            }
        }

        /// <summary>
        /// Applies the inverse projection, and updates the GUI.
        /// </summary>
        /// <param name="sender">Sender of the Event (UnprojectButton).</param>
        /// <param name="e">Event Arguments.</param>
        private void UnprojectButton_Click(object sender, RoutedEventArgs e)
        {
            SqlProjection p = this.GetProjection();
            SqlGeometry sqlGeometry = SqlGeometry.Parse(this.ProjectedTextBox.Text);
            sqlGeometry.STSrid = 4326; // WGS84
            SqlGeography sqlGeography = p.Unproject(sqlGeometry);
            this.GeometryTextBox.Text = sqlGeography.STAsText().ToSqlString().Value;
            this.DrawButton_Click(null, null);
        }

        /// <summary>
        /// Evaluates the input and initializes a SqlProjection.
        /// </summary>
        /// <returns>A SqlProjection.</returns>
        private SqlProjection GetProjection()
        {
            SqlProjection p;

            // Assign Parameter Values
            double param1 = double.Parse(this.Param1TextBox.Text);
            double param2 = double.Parse(this.Param2TextBox.Text);
            double param3 = double.Parse(this.Param3TextBox.Text);
            double param4 = double.Parse(this.Param4TextBox.Text);
            double param5 = double.Parse(this.Param5TextBox.Text);

            // Initialize sqlProjection
            switch (((ListBoxItem)this.ProjectionTypesCombo.SelectedItem).Content.ToString())
            {
                case "Albers Equal Area":
                    p = SqlProjection.AlbersEqualArea(param1, param2, param3, param4);
                    break;
                case "EquiRectangular":
                    param2 = double.Parse(this.Param2TextBox.Text);
                    p = SqlProjection.Equirectangular(param1, param2);
                    break;
                case "Gnomonic":
                    param2 = double.Parse(this.Param2TextBox.Text);
                    p = SqlProjection.Gnomonic(param1, param2);
                    break;
                case "Lambert Conformal Conic":
                    param2 = double.Parse(this.Param2TextBox.Text);
                    p = SqlProjection.LambertConformalConic(param1, param2, param3, param4);
                    break;
                case "Mercator":
                    p = SqlProjection.Mercator(param1);
                    break;
                case "Oblique Mercator":
                    p = SqlProjection.ObliqueMercator(param1, param2, param3, param4, param5);
                    break;
                case "Transverse Mercator":
                    p = SqlProjection.TranverseMercator(param1);
                    break;
                default:
                    p = SqlProjection.Null;
                    break;
            }

            return p;
        }

        /// <summary>
        /// Updates the GUI when the user selects a projection.
        /// </summary>
        /// <param name="sender">Sender of the Event (ProjectionTypesCombo).</param>
        /// <param name="e">Event Arguments.</param>
        private void ProjectionTypesCombo_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            switch (((ListBoxItem)this.ProjectionTypesCombo.SelectedItem).Content.ToString())
            {
                case "Albers Equal Area":
                    this.Param2TextBlock.Visibility = Visibility.Visible;
                    this.Param2TextBlock.Text = "Latitude: ";
                    this.Param2TextBox.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Text = "Parallel 1: ";
                    this.Param3TextBox.Visibility = Visibility.Visible;
                    this.Param4TextBlock.Visibility = Visibility.Visible;
                    this.Param4TextBlock.Text = "Parallel 2: ";
                    this.Param4TextBox.Visibility = Visibility.Visible;
                    this.Param5TextBlock.Visibility = Visibility.Collapsed;
                    this.Param5TextBox.Visibility = Visibility.Collapsed;
                    break;
                case "EquiRectangular":
                    this.Param2TextBlock.Visibility = Visibility.Visible;
                    this.Param2TextBlock.Text = "Parallel: ";
                    this.Param2TextBox.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Visibility = Visibility.Collapsed;
                    this.Param3TextBox.Visibility = Visibility.Collapsed;
                    this.Param4TextBlock.Visibility = Visibility.Collapsed;
                    this.Param4TextBox.Visibility = Visibility.Collapsed;
                    this.Param5TextBlock.Visibility = Visibility.Collapsed;
                    this.Param5TextBox.Visibility = Visibility.Collapsed;
                    break;
                case "Gnomonic":
                    this.Param2TextBlock.Visibility = Visibility.Visible;
                    this.Param2TextBlock.Text = "Latitude: ";
                    this.Param2TextBox.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Visibility = Visibility.Collapsed;
                    this.Param3TextBox.Visibility = Visibility.Collapsed;
                    this.Param4TextBlock.Visibility = Visibility.Collapsed;
                    this.Param4TextBox.Visibility = Visibility.Collapsed;
                    this.Param5TextBlock.Visibility = Visibility.Collapsed;
                    this.Param5TextBox.Visibility = Visibility.Collapsed;
                    break;
                case "Lambert Conformal Conic":
                    this.Param2TextBlock.Visibility = Visibility.Visible;
                    this.Param2TextBlock.Text = "Latitude: ";
                    this.Param2TextBox.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Text = "Fi 1: ";
                    this.Param3TextBox.Visibility = Visibility.Visible;
                    this.Param4TextBlock.Visibility = Visibility.Visible;
                    this.Param4TextBlock.Text = "Fi 2: ";
                    this.Param4TextBox.Visibility = Visibility.Visible;
                    this.Param5TextBlock.Visibility = Visibility.Collapsed;
                    this.Param5TextBox.Visibility = Visibility.Collapsed;
                    break;
                case "Mercator":
                    this.Param2TextBlock.Visibility = Visibility.Collapsed;
                    this.Param2TextBox.Visibility = Visibility.Collapsed;
                    this.Param3TextBlock.Visibility = Visibility.Collapsed;
                    this.Param3TextBox.Visibility = Visibility.Collapsed;
                    this.Param4TextBlock.Visibility = Visibility.Collapsed;
                    this.Param4TextBox.Visibility = Visibility.Collapsed;
                    this.Param5TextBlock.Visibility = Visibility.Collapsed;
                    this.Param5TextBox.Visibility = Visibility.Collapsed;
                    break;
                case "Oblique Mercator":
                    this.Param2TextBlock.Visibility = Visibility.Visible;
                    this.Param2TextBlock.Text = "Fi 1: ";
                    this.Param2TextBox.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Visibility = Visibility.Visible;
                    this.Param3TextBlock.Text = "Lambda 1: ";
                    this.Param3TextBox.Visibility = Visibility.Visible;
                    this.Param4TextBlock.Visibility = Visibility.Visible;
                    this.Param4TextBlock.Text = "Fi 2: ";
                    this.Param4TextBox.Visibility = Visibility.Visible;
                    this.Param5TextBlock.Visibility = Visibility.Visible;
                    this.Param5TextBlock.Text = "Lambda 2: ";
                    this.Param5TextBox.Visibility = Visibility.Visible;
                    break;
                case "Transverse Mercator":
                    this.Param2TextBlock.Visibility = Visibility.Collapsed;
                    this.Param2TextBox.Visibility = Visibility.Collapsed;
                    this.Param3TextBlock.Visibility = Visibility.Collapsed;
                    this.Param3TextBox.Visibility = Visibility.Collapsed;
                    this.Param4TextBlock.Visibility = Visibility.Collapsed;
                    this.Param4TextBox.Visibility = Visibility.Collapsed;
                    this.Param5TextBlock.Visibility = Visibility.Collapsed;
                    this.Param5TextBox.Visibility = Visibility.Collapsed;
                    break;
                default:
                    break;
            }
        }
    }
}
