﻿//-----------------------------------------------------------------------
// <copyright file="SqlGeometryExtensions.cs" company="U2U Consult">
//     http://www.U2UConsult.com
// </copyright>
// <summary>Defines the SqlGeometryExtensions class.</summary>
//-----------------------------------------------------------------------

namespace U2UConsult.DockOfTheBay
{
    using System.Windows;
    using System.Windows.Media;
    using System.Windows.Shapes;
    using Microsoft.SqlServer.Types;

    /// <summary>
    /// Extension Methods for displaying a SqlGeometry instance in WPF.
    /// </summary>
    public static class SqlGeometryExtensions
    {
        /// <summary>
        /// Translates a SqlGeometry into a Systems.Windows.Media.Geometry.
        /// </summary>
        /// <param name="sqlGeometry">The SqlGeometry instance to be visualized.</param>
        /// <returns>A Media.Geometry representing the sqlGeometry.</returns>
        /// <remarks>Uses a default radius for Points.</remarks>
        public static Geometry AsWpfGeometry(this SqlGeometry sqlGeometry)
        {
            // Make OGC Compliant
            if (!sqlGeometry.STIsValid())
            {
                sqlGeometry = sqlGeometry.MakeValid();
            }

            // Calculate default radius for points.
            SqlGeometry boundingBox = sqlGeometry.STEnvelope();
            double radius = boundingBox.STLength().Value / 400;

            return sqlGeometry.AsWpfGeometry(radius);
        }

        /// <summary>
        /// Translates a SqlGeometry into a Systems.Windows.Media.Geometry.
        /// </summary>
        /// <param name="sqlGeometry">The SqlGeometry instance to be visualized.</param>
        /// <param name="radius">Radius of the circles representing a Point.</param>
        /// <returns>A Media.Geometry representing the sqlGeometry.</returns>
        public static Geometry AsWpfGeometry(this SqlGeometry sqlGeometry, double radius)
        {
            // Make OGC Compliant
            if (!sqlGeometry.STIsValid())
            {
                sqlGeometry = sqlGeometry.MakeValid();
            }

            PathGeometry result = new PathGeometry();

            switch (sqlGeometry.STGeometryType().Value.ToLower())
            {
                case "point":

                    // Return a small circle
                    return sqlGeometry.STBuffer(radius).AsWpfGeometry(radius);

                case "polygon":

                    // A Spacial Polygon is a collection of Rings
                    // A Ring is a Closed LineString
                    // So, return a PathFigure for each Ring

                    // Outer Ring
                    result.Figures.Add(LineStringToWpfGeometry(sqlGeometry.STExteriorRing()));

                    // Inner Rings
                    for (int i = 1; i <= sqlGeometry.STNumInteriorRing(); i++)
                    {
                        result.Figures.Add(LineStringToWpfGeometry(sqlGeometry.STInteriorRingN(i)));
                    }

                    return result;

                case "linestring":

                    // Return a PathFigure
                    result.Figures.Add(LineStringToWpfGeometry(sqlGeometry));
                    return result;

                case "multipoint":
                case "multilinestring":
                case "multipolygon":
                case "geometrycollection":

                    // Return a Group of Geometries
                    GeometryGroup geometryGroup = new GeometryGroup();

                    for (int i = 1; i <= sqlGeometry.STNumGeometries().Value; i++)
                    {
                        geometryGroup.Children.Add(sqlGeometry.STGeometryN(i).AsWpfGeometry(radius));
                    }

                    return geometryGroup;

                default:

                    // Unrecognized Type
                    // Return an empty Geometry
                    return Geometry.Empty;
            }
        }

        /// <summary>
        /// Translates a SqlGeometry into a Systems.Windows.Shapes.Path.
        /// </summary>
        /// <param name="sqlGeometry">The SqlGeometry instance to be visualized.</param>
        /// <returns>A Shapes.Path representing the sqlGeometry.</returns>
        public static Path AsPath(this SqlGeometry sqlGeometry)
        {
            Path path = new Path();
            path.Data = sqlGeometry.AsWpfGeometry();

            // Flip Y-coordinate
            // Origin of a map is usually at the bottom left
            path.LayoutTransform = new ScaleTransform(1, -1);

            return path;
        }

        /// <summary>
        /// Translates a LineString or a single Polygon Ring to a PathFigure.
        /// </summary>
        /// <param name="sqlGeometry">The SqlGeometry instance to be visualized.</param>
        /// <returns>A Media.PathFigure representing the sqlGeometry.</returns>
        private static PathFigure LineStringToWpfGeometry(SqlGeometry sqlGeometry)
        {
            PathFigure result = new PathFigure();

            result.StartPoint = new Point(
                sqlGeometry.STPointN(1).STX.Value,
                sqlGeometry.STPointN(1).STY.Value);

            for (int i = 1; i <= sqlGeometry.STNumPoints(); i++)
            {
                LineSegment lineSegment = new LineSegment();
                lineSegment.Point = new Point(
                    sqlGeometry.STPointN(i).STX.Value,
                    sqlGeometry.STPointN(i).STY.Value);
                result.Segments.Add(lineSegment);
            }

            return result;
        }
    }
}